/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "menu.h"

void menuInit(menu_t *menu) {
  gridInit(&menu->grid);
  menu->grid.user = menu;

  menu->cursorX = 0;
  menu->cursorY = 0;
  menu->selected = 0;
  menu->user = NULL;
  menu->onSelect = NULL;
}

void menuUpdate(menu_t *menu, engine_t *engine) {
  uint8_t x, y, i, j, cx, cy;
  gridchild_t *current;
  gridchildbreakpoint_t *currentbp;
  gridchild_t *item;
  gridchildbreakpoint_t *itembp;


  current = menu->grid.children + menu->selected;
  currentbp = gridChildGetBreakpoint(current, menu->grid.breakpointCurrent);
  
  if(inputIsPressed(&engine->input, INPUT_ACCEPT)) {
    if(menu->onSelect != NULL) menu->onSelect(menu->user, menu->selected);
    return;
  }
  

  // Handle press binds.
  if(inputIsPressed(&engine->input, INPUT_DOWN)) {
    x = 0;
    y = 1;
  } else if(inputIsPressed(&engine->input, INPUT_UP)) {
    x = 0;
    y = -1;
  } else if(inputIsPressed(&engine->input, INPUT_LEFT)) {
    x = -1;
    y = 0;
  } else if(inputIsPressed(&engine->input, INPUT_RIGHT)) {
    x = 1;
    y = 0;
  } else {
    x = 0;
    y = 0;
  }

  
  // Update cursor positions
  if(x != 0 || y != 0) {
    if(x > 0) {
      cx = (currentbp->x + currentbp->columns - 1) + x;
    } else if(x < 0) {
      cx = currentbp->x + x;
    } else {
      cx = menu->cursorX;
    }

    if(y > 0) {
      cy = (currentbp->y + currentbp->rows - 1) + y;
    } else if(y < 0) {
      cy = currentbp->y + y;
    } else {
      cy = menu->cursorY;
    }
    
    // Get the item selected
    j = GRID_CHILD_COUNT;
    for(i = 0; i < menu->grid.childCount; i++) {
      if(i == menu->selected) continue;
      item = menu->grid.children + i;
      itembp = gridChildGetBreakpoint(item, menu->grid.breakpointCurrent);

      if(
        itembp->x > cx || (itembp->x + itembp->columns - 1) < cx ||
        itembp->y > cy || (itembp->y + itembp->rows - 1) < cy
      ) continue;

      j = i;
      break;
    }

    // Was a target found?
    if(j == GRID_CHILD_COUNT) return;
    menu->cursorX = cx;
    menu->cursorY = cy;
    menu->selected = j;
  }
}